exception FilePathBaseFilenameRelative
exception FilePathRelativeUnreducable
exception FilePathUnrecognizedOS of string
exception FilePathFilenameMultiple
exception FilePathEmpty
exception FilePathNoExtension
exception FilePathInvalidFilename
module type OS_SPECIFICATION =
  sig
    val dir_writer : FilePath_type.filename_part list -> string
    val dir_reader : Lexing.lexbuf -> FilePath_type.filename_part list
    val path_writer : FilePath_type.filename list -> string
    val path_reader : Lexing.lexbuf -> FilePath_type.filename list
  end
module type PATH_SPECIFICATION =
  sig
    type filename
    type extension
    val string_of_filename : filename -> string
    val filename_of_string : string -> filename
    val make_filename : string list -> filename
    val is_subdir : filename -> filename -> bool
    val is_updir : filename -> filename -> bool
    val compare : filename -> filename -> int
    val basename : filename -> filename
    val dirname : filename -> filename
    val concat : filename -> filename -> filename
    val reduce : filename -> filename
    val make_absolute : filename -> filename -> filename
    val make_relative : filename -> filename -> filename
    val reparent : filename -> filename -> filename -> filename
    val identity : filename -> filename
    val is_valid : filename -> bool
    val is_relative : filename -> bool
    val is_implicit : filename -> bool
    val is_current : filename -> bool
    val is_parent : filename -> bool
    val chop_extension : filename -> filename
    val get_extension : filename -> extension
    val check_extension : filename -> extension -> bool
    val add_extension : filename -> extension -> filename
    val extension_of_string : string -> extension
    val string_of_extension : extension -> string
    val string_of_path : filename list -> string
    val path_of_string : string -> filename list
    val current_dir : filename
    val parent_dir : filename
  end
module type PATH_STRING_SPECIFICATION =
  sig
    type filename = string
    type extension
    val string_of_filename : filename -> string
    val filename_of_string : string -> filename
    val make_filename : string list -> filename
    val is_subdir : filename -> filename -> bool
    val is_updir : filename -> filename -> bool
    val compare : filename -> filename -> int
    val basename : filename -> filename
    val dirname : filename -> filename
    val concat : filename -> filename -> filename
    val reduce : filename -> filename
    val make_absolute : filename -> filename -> filename
    val make_relative : filename -> filename -> filename
    val reparent : filename -> filename -> filename -> filename
    val identity : filename -> filename
    val is_valid : filename -> bool
    val is_relative : filename -> bool
    val is_implicit : filename -> bool
    val is_current : filename -> bool
    val is_parent : filename -> bool
    val chop_extension : filename -> filename
    val get_extension : filename -> extension
    val check_extension : filename -> extension -> bool
    val add_extension : filename -> extension -> filename
    val extension_of_string : string -> extension
    val string_of_extension : extension -> string
    val string_of_path : filename list -> string
    val path_of_string : string -> filename list
    val current_dir : filename
    val parent_dir : filename
  end
module type META_PATH_SPECIFICATION =
  functor (OsOperation : OS_SPECIFICATION) -> PATH_SPECIFICATION
module type META_PATH_STRING_SPECIFICATION =
  functor (PathOperation : PATH_SPECIFICATION) -> PATH_STRING_SPECIFICATION
module GenericPath : META_PATH_SPECIFICATION
module GenericStringPath : META_PATH_STRING_SPECIFICATION
module AbstractDefaultPath : PATH_SPECIFICATION
module DefaultPath : PATH_STRING_SPECIFICATION
module AbstractUnixPath : PATH_SPECIFICATION
module UnixPath : PATH_STRING_SPECIFICATION
module AbstractMacOSPath : PATH_SPECIFICATION
module MacOSPath : PATH_STRING_SPECIFICATION
module AbstractWin32Path : PATH_SPECIFICATION
module Win32Path : PATH_STRING_SPECIFICATION
module AbstractCygwinPath : PATH_SPECIFICATION
module CygwinPath : PATH_STRING_SPECIFICATION
