type filename = string
exception SizeInvalid
exception FileDoesntExist
exception RecursiveLink of filename
exception RmDirNotEmpty of filename
exception MkdirMissingComponentPath of filename
exception MkdirDirnameAlreadyUsed of filename
exception CpCannotCopy of filename
exception CpNoSourceFile of filename
exception CpCannotCopyFilesToFile of filename list * filename
exception CpCannotCopyDir of filename
exception MvNoSourceFile
type action_link =
    Follow
  | Skip
  | SkipInform of (filename -> unit)
  | AskFollow of (filename -> bool)
type interactive = Force | Ask of (filename -> bool)
type size =
    TB of float
  | GB of float
  | MB of float
  | KB of float
  | B of float
type kind = Dir | File | Dev_char | Dev_block | Link | Fifo | Socket
type base_permission = {
  sticky : bool;
  exec : bool;
  write : bool;
  read : bool;
}
type permission = {
  user : base_permission;
  group : base_permission;
  other : base_permission;
}
type stat = {
  kind : kind;
  is_link : bool;
  permission : permission;
  size : size;
  owner : int;
  group_owner : int;
  access_time : float;
  modification_time : float;
  creation_time : float;
}
type test_file =
    Is_dev_block
  | Is_dev_char
  | Is_dir
  | Exists
  | Is_file
  | Is_set_group_ID
  | Has_sticky_bit
  | Is_link
  | Is_pipe
  | Is_readable
  | Is_writeable
  | Size_not_null
  | Size_bigger_than of size
  | Size_smaller_than of size
  | Size_equal_to of size
  | Size_fuzzy_equal_to of size
  | Is_socket
  | Has_set_user_ID
  | Is_exec
  | Is_owned_by_user_ID
  | Is_owned_by_group_ID
  | Is_newer_than of filename
  | Is_older_than of filename
  | Is_newer_than_date of float
  | Is_older_than_date of float
  | And of test_file * test_file
  | Or of test_file * test_file
  | Not of test_file
  | Match of string
  | True
  | False
  | Has_extension of string
  | Has_no_extension
  | Is_parent_dir
  | Is_current_dir
  | Basename_is of filename
  | Dirname_is of filename
  | Custom of (filename -> bool)
module type OPERATION_REGEXP =
  sig type t val compile : string -> t val test : t -> string -> bool end
module type FILE_UTILS =
  sig
    val ls : filename -> filename list
    val filter : test_file -> filename list -> filename list
    val test : test_file -> filename -> bool
    val which : ?path:filename list -> filename -> filename
    val mkdir : ?parent:bool -> ?mode:int -> filename -> unit
    val touch : ?create:bool -> filename -> unit
    val find :
      ?follow:action_link ->
      test_file -> filename -> ('a -> filename -> 'a) -> 'a -> 'a
    val rm : ?force:interactive -> ?recurse:bool -> filename list -> unit
    val cp :
      ?follow:action_link ->
      ?force:interactive ->
      ?recurse:bool -> filename list -> filename -> unit
    val mv : ?force:interactive -> filename -> filename -> unit
    val readlink : filename -> filename
    val pwd : unit -> filename
    val cmp : ?skip1:int -> filename -> ?skip2:int -> filename -> int option
    val du : filename list -> size * (filename * size) list
    val stat : filename -> stat
  end
module type META_FILE_UTILS =
  functor (OperationRegexp : OPERATION_REGEXP) -> FILE_UTILS
val permission_of_int : int -> permission
val int_of_permission : permission -> int
val size_convert_down : size -> size
val size_convert_up : size -> size
val size_compare_unit : size -> size -> int
val size_to_same_unit : size -> size -> size
val size_to_To : size -> size
val size_to_Go : size -> size
val size_to_Mo : size -> size
val size_to_Ko : size -> size
val size_to_O : size -> size
val size_apply_operation : (float -> float -> float) -> size -> size -> size
val size_compare : ?fuzzy:bool -> size -> size -> int
val size_add : size -> size -> size
val size_sub : size -> size -> size
val string_of_size : ?fuzzy:bool -> size -> string
module GenericUtil : META_FILE_UTILS
module StrUtil : FILE_UTILS
